C
Code by Michael Kupferschmid
C
      SUBROUTINE ADDACC(P, XYSUM )
C     This routine adds P to the extra-precision accumulator XYSUM.
C     It must be compiled with optimization turned off.
C
C     variable  meaning
C     --------  -------
C     DABS      Fortran function returns |REAL*8|
C     P         quantity to be added to the accumulator
C     U         the larger in absolute value of P and XYSUM
C     V         the smaller in absolute value of P and XYSUM
C     XYSUM     the accumulator
C     Z         most significant part of sum
C     ZZ        least significant part of sum
C
C     formal parameters
      REAL*8 P,XYSUM(2)
C
C     local variables
      REAL*8 U,V,Z,ZZ
C
C ------------------------------------------------------------------
C
C     put the larger quantity in U and the smaller in V
      IF(DABS(XYSUM(1)) .LT. DABS(P)) THEN
         U=P
         V=XYSUM(1)
      ELSE
         U=XYSUM(1)
         V=P
      ENDIF
C
C     find the sum, imprecisely
      Z=U+V
C
C     compute the error that was made by rounding U+V to REAL*8
      ZZ=(U-Z)+V
C
C     add to it the least significant part of the accumulator
      ZZ=ZZ+XYSUM(2)
C
C     that might be enough to increase the most significant part
      XYSUM(1)=Z+ZZ
C
C     make the least significant part of accumulator what is left
      XYSUM(2)=(Z-XYSUM(1))+ZZ
C
      RETURN
      END
