C
Code by Michael Kupferschmid
C
      FUNCTION DAYOF(DATE)
C     This routine returns the number of the day corresponding to
C     DATE, counting 01 Jan 70 as day 1, or 0 if an error occurs.
C     The format of DATE is dd Mmm yy.  Values of "yy" less than
C     "70" are assumed to be years 2000...2069 CE.
C
C     variable  meaning
C     --------  -------
C     COPY      copy of DATE for overlays
C     DATE      date whose day number is to be found 
C     DAY       day as characters "dd"
C     DAYS      numbers of days in the months, not a leap year
C     DTB       routine converts numerals to an INTEGER*4 
C     K         index on the characters of DATE and COPY
C     LEAPYR    function returns T if argument year is a leap year
C     MONTH     month as characters "Mmm"
C     MOS       the names of the months as characters "Mmm"
C     N         index on years or months
C     NDAY      number of the day "dd" in the month as INTEGER*4
C     NMONTH    number of the month "Mmm" in the year as INTEGER*4
C     NYEAR     number of the year "yy" as INTEGER*4
C     RC        return code from DTB; 0 => ok
C     YEAR      year as characters "yy"
C
C     formal parameters
      INTEGER*4 DAYOF
      CHARACTER*1 DATE(9)
C
C     prepare overlays to extract DAY, MONTH, and YEAR
      CHARACTER*1 COPY(9)
      CHARACTER*2 DAY
      EQUIVALENCE(COPY(1),DAY)
      CHARACTER*3 MONTH
      EQUIVALENCE(COPY(4),MONTH)
      CHARACTER*2 YEAR
      EQUIVALENCE(COPY(8),YEAR)
C
C     local variables
      LOGICAL*4 LEAPYR
      CHARACTER*3 MOS(12)/'Jan','Feb','Mar','Apr','May','Jun',
     ;                    'Jul','Aug','Sep','Oct','Nov','Dec'/
      INTEGER*4 DAYS(12)/31,28,31,30,31,30,31,31,30,31,30,31/
      INTEGER*4 RC
C
C ------------------------------------------------------------------
C
C     copy the date string into the overlay
      DO 1 K=1,9
           COPY(K)=DATE(K)
    1 CONTINUE
C
C     get the number of the year
      CALL DTB(YEAR,2,NYEAR,RC)
      IF(RC.NE.0) GO TO 2
      IF(NYEAR.LT.0) GO TO 2
      IF(NYEAR.GE.70) THEN
C        the year is 1970...1999
         NYEAR=NYEAR+1900
      ELSE
C        the year is 2000...2069
         NYEAR=NYEAR+2000
      ENDIF
C
C     add up the days in the years 1970...NYEAR-1
      DAYOF=0
      IF(NYEAR.EQ.1970) GO TO 3
      DO 4 N=1970,NYEAR-1
           DAYOF=DAYOF+365
           IF(LEAPYR(N)) DAYOF=DAYOF+1
    4 CONTINUE
C
C     find the number of the month
    3 DO 5 N=1,12
           IF(MONTH.NE.MOS(N)) GO TO 5
           NMONTH=N
           GO TO 6  
    5 CONTINUE 
C     the month name was not in the list
      GO TO 2
C
C     add in the days in the months 1...NMONTH-1
    6 IF(NMONTH.EQ.1) GO TO 7
      DO 8 N=1,NMONTH-1
           DAYOF=DAYOF+DAYS(N)
           IF(N.NE.2) GO TO 8
C          February; check for the extra day
           IF(LEAPYR(NYEAR)) DAYOF=DAYOF+1
    8 CONTINUE
C
C     get and check the number of the day
    7 CALL DTB(DAY,2,NDAY,RC)
      IF(RC.NE.0) GO TO 2
      IF(NDAY.LT.1) GO TO 2
      IF(NMONTH.NE.2) THEN
        IF(NDAY.GT.DAYS(NMONTH)) GO TO 2
      ELSE
C       February; check against 28 or 29 days
        IF(.NOT.LEAPYR(NYEAR)) THEN
           IF(NDAY.GT.28) GO TO 2
        ELSE
           IF(NDAY.GT.29) GO TO 2 
        ENDIF
      ENDIF
C
C     add in the number of days elapsed so far this month
      DAYOF=DAYOF+NDAY
      RETURN
C
C     an error occurred; return the value zero
    2 DAYOF=0
      RETURN
      END
