C
Code by Michael Kupferschmid
C
      FUNCTION DDOTQ(X,Y,N)
C     This routine computes the dot product of X with Y,
C     using extra-precision accumulation.
C
C     variable  meaning
C     --------  -------
C     J         index on the elements of X and Y
C     MPYACC    routine does extra-precision multiply and accumulate
C     N         number of elements in X and Y
C     X         one of the vectors in the dot product
C     XYSUM     extra-precision result
C     Y         the other vector in the dot product
C
C     formal parameters
      REAL*8 DDOTQ,X(N),Y(N)
C
C     local variable
      REAL*8 XYSUM(2)
C
C ------------------------------------------------------------------
C
C     check for a sensible value of N
      DDOTQ=0.D0
      IF(N.LE.0) RETURN
C
C     accumulate the product at extended precision
      XYSUM(1)=0.D0
      XYSUM(2)=0.D0
      DO 1 J=1,N
           CALL MPYACC(X(J),Y(J), XYSUM )
    1 CONTINUE
C
C     return a double-precision answer
      DDOTQ=XYSUM(1)+XYSUM(2)
      RETURN
      END
