C
Code by Michael Kupferschmid
C
      SUBROUTINE FRDR8V(NUNIT,X,LDX, N,RC)
C     This routine reads a vector of REAL*8s from a given unit.
C
C     RC  meaning
C     --  -------
C     0   all went well
C     1   end of file
C     2   parameter error or read error
C     3   too many data items found in line
C     4   line too long for buffer
C     5   the line is a comment
C
C     variable  meaning
C     --------  -------
C     BLANK     the character ' ' as an integer
C     BNKCMT    routine blanks out a comment in a string
C     COMMA     the character , as an integer
C     DEV       the string '/dev/'
C     FYLE      name to which unit 5 is attached; /dev/* => keyboard
C     K         index on the characters of TEXT
C     L         remaining length of the string read in
C     LDX       dimension of X
C     LENGTH    function gives index of last nonblank in string
C     LINE      text line read in
C     N         the number of values found on the line
C     NAM       the first 5 characters of FYLE
C     NUNIT     logical unit number from which to read the line
C     RC        return code; see table above
C     SHIFTL    routine removes leading blanks from a string
C     SKIP      T => the whole line is a comment
C     TAB       the tab character
C     TEXT      LINE as a string of single characters
C     X         the REAL*8 vector returned
C
C     formal parameters
      REAL*8 X(LDX)
      INTEGER*4 RC
C
C     prepare to read and process input lines
      CHARACTER*1024 LINE
      INTEGER*1 TEXT(1024)
      EQUIVALENCE(LINE,TEXT(1))
      INTEGER*1 BLANK/Z'20'/,COMMA/Z'2C'/,TAB/Z'09'/
      LOGICAL*4 SKIP
C
C     prepare to deal with end of file
      CHARACTER*24 FYLE
      CHARACTER*5 NAM
      EQUIVALENCE(NAM,FYLE)
      CHARACTER*5 DEV/'/dev/'/
C
C ------------------------------------------------------------------
C
C     assume there will be no errors
      RC=0
C
C     check for sensible input paramters
      IF(LDX.LE.0) RC=2
      IF(NUNIT.LE.0 .OR. NUNIT.GT.99 .OR. NUNIT.EQ.6) RC=2
      IF(RC.NE.0) RETURN
C
C     to begin with no values have been read
      DO 1 N=1,LDX
           X(N)=0.D0
    1 CONTINUE
      N=0
C
C     read a single line as characters
      READ(NUNIT,900,END=2,ERR=3) LINE
  900 FORMAT(A1024)
      GO TO 4
    2 RC=1
      INQUIRE(UNIT=NUNIT,NAME=FYLE)
      IF(NUNIT.EQ.5 .AND. NAM.EQ.DEV) THEN
C        standard-in is the keyboard; reopen it for more input
         CLOSE(UNIT=5)
         OPEN(UNIT=5,FILE='/dev/tty')
      ENDIF
      RETURN
    3 RC=2
      RETURN
    4 L=LENGTH(TEXT,1024)
      IF(L.GT.1023) THEN
         RC=4
         RETURN
      ENDIF
C
C     blank out comments
      CALL BNKCMT(TEXT,L,SKIP)
      IF(SKIP) THEN
         RC=5
         RETURN
      ENDIF
      IF(L.EQ.0) RETURN
C
C     change any commas or tabs to blanks
      DO 5 K=1,L
           IF(TEXT(K).EQ.COMMA .OR. TEXT(K).EQ.TAB) TEXT(K)=BLANK
    5 CONTINUE
C
C     shift the string left to put the next token first
    9 CALL SHIFTL(TEXT,L)
      L=LENGTH(TEXT,L)
      IF(L.EQ.0) RETURN
C
C     not done; read the next value
      N=N+1
      IF(N.GT.LDX) THEN
         RC=3
         RETURN
      ENDIF
      READ(LINE,*,ERR=6) X(N)
      GO TO 7
    6 RC=2
      RETURN
C
C     blank out the value just read
    7 DO 8 K=1,L
           IF(TEXT(K).EQ.BLANK) GO TO 9
           TEXT(K)=BLANK
    8 CONTINUE
      GO TO 9
      END
