C
Code by Michael Kupferschmid
C
      SUBROUTINE GETFIL(WATFOR,LWIN,SUGNAM,LSIN,NUNIT,ACCESS)
C     This routine attaches a file to a given unit.
C
C     ACCESS  operations to be performed
C     ------  --------------------------
C        1    read
C        2    write
C        3    read and write
C       -1    read unformatted
C       -2    write unformatted
C       -3    read and write unformatted
C
C     variable  meaning
C     --------  -------
C     ACCESS    access required to the file; see above
C     ARG       a command-line token as a CHARACTER*68
C     DTB       routine converts numerals to an integer
C     EROR      error message text written out
C     EROR1     "The file ? is to be read, but it does not exist."
C     EROR2     "The file ? cannot be opened; IOSTAT=?."
C     FILNAM    name of the file to open
C     FNPR      filename prompt string written out
C     FNPRT     filename prompt string template
C     GETARG    system routine gets a given command-line token
C     IABS      Fortran function gives |INTEGER*4|
C     IARGC     system function returns # of command-line tokens
C     INTINS    routine inserts an integer in a character string
C     IPAR      index on the command-line tokens
C     K         index on the characters of a string
C     LE        finished length of EROR
C     LENGTH    function returns index of last nonblank in a string
C     LF        number of characters in FILNAM
C     LN        number of characters in a command-line unit number
C     LONGNM    FILNAM as a CHARACTER*65
C     LP        finished length of FNPR
C     LQ        finished length of QUES
C     LS        number of characters of SUGNAM actually used
C     LSIN      number of characters in SUGNAM
C     LW        number of characters of WATFOR actually used
C     LWIN      number of characters in WATFOR
C     MAX0      Fortran function gives larger of INTEGER*4s
C     N         a unit number assigned on the command line
C     NPAR      number of tokens appearing on the command line
C     NUNIT     unit number to assign
C     OK        T => unit is already assigned or file exists
C     PAR       a command-line token as a vector of CHARACTER*1s
C     PROMPT    routine prompts for input from the terminal
C     QUERY     function asks a yes-or-no question
C     QUES      question text written out
C     QUEST     "The file ? is to be written, but it exists; ok?"
C     RC        return code from DTB or OPEN
C     PACKL     routine removes blanks from a string
C     STRINS    routine inserts a string in a template
C     SUGNAM    suggested or default filename
C     UNLINK    system routine destroys a file
C     WATFOR    string telling what the file is to be used for
C
C     formal parameters
      CHARACTER*1 WATFOR(LWIN),SUGNAM(LSIN)
      INTEGER*4 ACCESS
C
C     prepare to check for command-line unit assignments
      CHARACTER*68 ARG
      CHARACTER*1 PAR(68)
      EQUIVALENCE(ARG,PAR)
      INTEGER*4 RC
C
C     prepare to prompt for and read a filename
      CHARACTER*1 FNPRT(78)/'N','a','m','e',' ','o','f',' ',70*' '/
      CHARACTER*1 FNPR(78),FILNAM(65)
      CHARACTER*65 LONGNM
      EQUIVALENCE(FILNAM,LONGNM)
C
C     prepare to ask about emptying a file to be written
      CHARACTER*1 QUES(113)
      CHARACTER*1 QUEST(113)/'T','h','e',' ','f','i','l','e',' ',
     ;                       '"', 65*' ','"',' ',
     ;                       'i','s',' ','t','o',' ','b','e',' ',
     ;                       'w','r','i','t','t','e','n',',',' ',
     ;                       'b','u','t',' ','i','t',' ',
     ;                       'e','x','i','s','t','s',';',' ',
     ;                       'o','k','?'/
      LOGICAL*4 OK,QUERY
C
C     prepare to report errors
      CHARACTER*1 EROR(114)
      CHARACTER*1 EROR1(114)/'T','h','e',' ','f','i','l','e',' ',
     ;                       '"',65*' ','"',' ',
     ;                       'i','s',' ','t','o',' ','b','e',' ',
     ;                       'r','e','a','d',',',' ','b','u','t',
     ;                       ' ','i','t',' ','d','o','e','s',' ',
     ;                       'n','o','t',' ','e','x','i','s','t',
     ;                       '.'/
      CHARACTER*1 EROR2(114)/'T','h','e',' ','f','i','l','e',' ',
     ;                       '"',65*' ','"',' ',
     ;                       'c','a','n','n','o','t',' ','b','e',
     ;                       ' ','o','p','e','n','e','d',';',' ',
     ;                       'I','O','S','T','A','T','=',
     ;                       11*' ','.'/
C
C ------------------------------------------------------------------
C
C     check for a sensible unit number
      IF(NUNIT.GE.0 .AND. NUNIT.LE.99 .AND.
     ;   NUNIT.NE.0 .AND. NUNIT.NE.5  .AND. NUNIT.NE.6) GO TO 1
      WRITE(0,991) NUNIT
  991 FORMAT('In GETFIL, bad unit number',I11,'; stopping.')
      STOP
C
C     see if the unit is already attached
    1 INQUIRE(UNIT=NUNIT,OPENED=OK)
      IF(OK) RETURN
C
C     check for a sensible access code
      IF(IABS(ACCESS).EQ.1 .OR.
     ;   IABS(ACCESS).EQ.2 .OR.
     ;   IABS(ACCESS).EQ.3) GO TO 2
      WRITE(0,992) ACCESS
  992 FORMAT('In GETFIL, bad access code',I11,'; stopping.')
      STOP
C
C     find actual input string lengths
    2 LW=MAX0(0,LWIN)
      LW=LENGTH(WATFOR,LW)
      LS=MAX0(0,LSIN)
      LS=LENGTH(SUGNAM,LS)
C
C     decide how the words will fit into the prompt string
      IF(LW.GT.0 .AND. LS.GT.0 .AND. LW+LS.GT.61) LS=0
      IF(LW.EQ.0 .AND. LS.GT.0 .AND.    LS.GT.62) LS=0
      IF(LW.GT.0 .AND. LS.EQ.0 .AND. LW   .GT.63) LW=63
C
C     construct the prompt string
C       what the file is for
        CALL STRINS(FNPRT,78,WATFOR,LW,9,0, FNPR,LP)
C
C       the word " file"
        CALL STRINS(FNPR,78,' file',5,LP+1,0, FNPR,LP)
C
C       the suggested filename, in square brackets
        IF(LS.LE.0) GO TO 3
        CALL STRINS(FNPR,78,' [',2,LP+1,0, FNPR,LP)
        CALL STRINS(FNPR,78,SUGNAM,LS,LP+1,0, FNPR,LP)
        CALL STRINS(FNPR,78,']',1,LP+1,0, FNPR,LP)
C
C       the colon
    3   CALL STRINS(FNPR,78,':',1,LP+1,0, FNPR,LP)
C
C     check for command-line parameters
      NPAR=IARGC()
      IF(NPAR.EQ.0) GO TO 4
C
C     examine each in search of an assignment like "3=fyle"
      DO 5 IPAR=1,NPAR
C          get the text of this command-line parameter
           CALL GETARG(IPAR,ARG)
C
C          look for an = in column 2 or 3
           LN=0
           IF(PAR(2).EQ.'=') LN=1
           IF(PAR(3).EQ.'=') LN=2
           IF(LN.EQ.0) GO TO 5
C
C          convert the numerals to an integer
           CALL DTB(PAR,LN, N,RC)
           IF(RC.NE.0) GO TO 5
C
C          is this assignment the one we are looking for?
           IF(N.NE.NUNIT) GO TO 5
           DO 6 K=1,65
                FILNAM(K)=PAR(LN+1+K)
    6      CONTINUE
           GO TO 7
    5 CONTINUE
C
C     prompt for and read the filename to try
    4 CALL PROMPT(FNPR,LP)
      READ(5,900,END=9) FILNAM
  900 FORMAT(65A1)
      GO TO 10
C     end-of-file means the user wants to quit
    9 STOP
C
C     check for a null or blank response
   10 CALL PACKL(FILNAM,65)
      IF(FILNAM(1).NE.' ') GO TO 7
C
C     zero-length line; use suggested name if one was given
      IF(LS.EQ.0) GO TO 4
      DO 11 K=1,LS
            FILNAM(K)=SUGNAM(K)
   11 CONTINUE
C
C     see if the named file already exists
    7 LF=LENGTH(FILNAM,65)
      INQUIRE(FILE=LONGNM,EXIST=OK)
C
C     if the file is only to be read, it must exist
      IF(IABS(ACCESS).NE.1 .OR. OK) GO TO 12
      CALL STRINS(EROR1,114,FILNAM,LF,11,2, EROR,LE)
      WRITE(0,993) (EROR(K),K=1,LE)
  993 FORMAT(114A1)
      GO TO 4
C
C     if the file is only to be written, maybe it shouldn't exist
   12 IF(IABS(ACCESS).NE.2 .OR. .NOT.OK) GO TO 13
      CALL STRINS(QUEST,113,FILNAM,LF,11,2, QUES,LQ)
      IF(.NOT.QUERY(QUES,LQ)) GO TO 4
      CALL UNLINK(LONGNM)
C
C     attach the named file to the specified unit
   13 IF(ACCESS.GT.0) THEN
         OPEN(UNIT=NUNIT,FILE=LONGNM,IOSTAT=RC)
      ELSE
         OPEN(UNIT=NUNIT,FILE=LONGNM,FORM='UNFORMATTED',IOSTAT=RC)
      ENDIF
      IF(RC.EQ.0) RETURN
C
C     the OPEN failed; report the error number and try again
      CALL INTINS(EROR2,114,RC,103,2, EROR,LE)
      CALL STRINS(EROR,114,FILNAM,LF,11,2, EROR,LE)
      WRITE(0,993) (EROR(K),K=1,LE)
      GO TO 4
      END
