C
Code by Michael Kupferschmid
C
      SUBROUTINE HB2HSH(MXHSTR,HSTR,MXHASH, NCH,CHASH,VHASH,RC,K)
C     This routine finds the hash corresponding to a Hebrew string.
C
C     RC  meaning
C     --  -------
C      0  all went well
C      1  HSTR has zero length, or MXHSTR=0
C      2  first character of HSTR is not \
C      3  premature end of HSTR
C      4  more than 10 characters in a vowel or trope name
C      5  vowel or trope name not recognized
C      6  more than MXHASH vowel or trope specifications in HSTR
C      7  more than 10 characters in a letter name
C      8  letter name not recognized
C      9  bad coordinate in a trope specification
C     10  more than MXHASH letters or coordinate pairs in HSTR
C     11  character after a letter is not \ or space
C
C     variable  meaning
C     --------  -------
C     CHASH     column of codes for letters in Hebrew string
C     HSTR      Hebrew string to be hashed
C     I         index on possible vowel/letter/trope names
C     IC        number of this character
C     IL        number of this letter
C     IT        number of this trope specification
C     IV        number of this vowel
C     J         index on characters in a letter or vowel name
C     K         index on the characters of HSTR
C     L         length of HSTR
C     LENGTH    function returns index of last nonblank in a string
C     LETTER    a letter name to find the number of
C     LTR       names of recognized letters
C     M         index on hash entries
C     MC        index on the characters in the string
C     MV        index on the vowels/hashes in the string
C     MXHASH    dimensioned length of CHASH and VHASH
C     MXHSTR    dimensioned length of HSTR
C     NCH       number of Hebrew characters in HSTR
C     NLTR      number of possible letters
C     NTRP      number of possible tropes
C     NVWL      number of possible vowels
C     RC        return code; see table above
C     TRP       names of recognized tropes
C     VHASH     column of codes for vowels/tropes in Hebrew string
C     VOWEL     a vowel or trope name to find the number of
C     VWL       names of recognized vowels
C     X         x-coordinate of a trope glyph
C     XTRDIG    routine extracts a coordinate from {x}{y}
C     XYSTOR    routine stores XY coordinates in a byte
C     Y         y-coordinate of a trope glyph
C
C     formal parameters
      CHARACTER*1 HSTR(MXHSTR)
      INTEGER*1 NCH,CHASH(MXHASH),VHASH(MXHASH),XYSTOR
      INTEGER*4 RC
C
C     local variables
      CHARACTER*10 VOWEL,LETTER
      INTEGER*4 X,Y
C
C     recognized vowel names
      PARAMETER(NVWL=14)
      CHARACTER*10 VWL(NVWL)/'hebrew    ',
     ;                       'chiriq    ',
     ;                       'tsere     ',
     ;                       'segol     ',
     ;                       'halfsegol ',
     ;                       'patach    ',
     ;                       'halfpatach',
     ;                       'qamats    ',
     ;                       'halfqamats',
     ;                       'awe       ',
     ;                       'qubbuts   ',
     ;                       'sheva     ',
     ;                       'lcholam   ',
     ;                       'rcholam   '/
C
C     recognized trope names
      PARAMETER(NTRP=3)
      CHARACTER*10 TRP(NTRP)/'dagesh    ',
     ;                       'meteg     ',
     ;                       'hebspc    '/
C
C     recognized letter names
      PARAMETER(NLTR=40)
      CHARACTER*10 LTR(NLTR)/'aleph     ',
     ;                       'bet       ',
     ;                       'vet       ',
     ;                       'gimel     ',
     ;                       'dalet     ',
     ;                       'hay       ',
     ;                       'vav       ',
     ;                       'ohvav     ',
     ;                       'oovav     ',
     ;                       'zayin     ',
     ;                       'khet      ',
     ;                       'tet       ',
     ;                       'yod       ',
     ;                       'endcoph   ',
     ;                       'coph      ',
     ;                       'endcaf    ',
     ;                       'caf       ',
     ;                       'lamed     ',
     ;                       'endmem    ',
     ;                       'mem       ',
     ;                       'endnun    ',
     ;                       'nun       ',
     ;                       'samech    ',
     ;                       'ayin      ',
     ;                       'endfay    ',
     ;                       'fay       ',
     ;                       'endpay    ',
     ;                       'pay       ',
     ;                       'endtsade  ',
     ;                       'tsade     ',
     ;                       'kuf       ',
     ;                       'resh      ',
     ;                       'shin      ',
     ;                       'sin       ',
     ;                       'sn        ',
     ;                       'taf       ',
     ;                       'dash      ',
     ;                       'space     ',
     ;                       'hash      ',
     ;                       'geresh    '/
C
C -----------------------------------------------------------------
C
C     initialize the columns of codes to all zeros
      DO 1 M=1,MXHASH
           CHASH(M)=0
           VHASH(M)=0
    1 CONTINUE
C     fill in character and vowel codes from right to left
      MC=MXHASH+1
      MV=MXHASH+1
C
C     the first character in the Hebrew string should be a backslash
      L=LENGTH(HSTR,MXHSTR)
      IF(L.EQ.0) THEN
         RC=1
         RETURN
      ENDIF
      K=1
      IF(HSTR(K).NE.'\') THEN
         RC=2
         RETURN
      ENDIF
C
C     extract the vowel or trope name up to {
   12 VOWEL='          '
      J=0
    2 K=K+1
      IF(K.GT.L) THEN
         RC=3
         RETURN
      ENDIF
      IF(HSTR(K).NE.'{') THEN
         J=J+1
         IF(J.GT.10) THEN
            RC=4
            RETURN
         ENDIF
         VOWEL(J:J)=HSTR(K)
         GO TO 2
      ENDIF
C
C     is this the name of a vowel?
      DO 3 I=1,NVWL
           IV=I
           IF(VOWEL.EQ.VWL(I)) GO TO 4
    3 CONTINUE
C
C     no; is it the name of a trope?
      DO 5 I=1,NTRP
           IT=NVWL+I
           IF(VOWEL.EQ.TRP(I)) GO TO 6
    5 CONTINUE
      RC=5
      RETURN
C
C     this is a vowel
C       code its name into the hash
    4   MV=MV-1
        IF(MV.EQ.0) THEN
           RC=6
           RETURN
        ENDIF
        VHASH(MV)=IV
C
C       the characters up to } should be a letter name
        LETTER='          '
        J=0
    7   K=K+1
        IF(K.GT.L) THEN
           RC=3
           RETURN
        ENDIF
        IF(HSTR(K).NE.'}') THEN
           J=J+1
           IF(J.GT.10) THEN
              RC=7
              RETURN
           ENDIF
           LETTER(J:J)=HSTR(K)
           GO TO 7
        ELSE
C          end of letter name; look up which one it is
           DO 8 I=1,NLTR
                IC=I
                IF(LETTER.EQ.LTR(I)) GO TO 9
    8      CONTINUE
           RC=8
           RETURN
        ENDIF

C       code this letter into the hash
    9   MC=MC-1
        IF(MC.EQ.0) THEN
           RC=10
           RETURN
        ENDIF
        CHASH(MC)=IC
      GO TO 10
C
C     this is a trope
C       code its name into the hash
    6   MV=MV-1
        IF(MV.EQ.0) THEN
           RC=6
           RETURN
        ENDIF
        VHASH(MV)=IT
C
C       code the coordinates of the glyph into the hash
        CALL XTRDIG(HSTR,L,K, X,RC)
        IF(RC.NE.0) RETURN
        K=K+1
        CALL XTRDIG(HSTR,L,K, Y,RC)
        IF(RC.NE.0) RETURN
        MC=MC-1
        IF(MC.EQ.0) THEN
           RC=10
           RETURN
        ENDIF
        CHASH(MC)=XYSTOR(X,Y)
C
C     next must be \ (starting next char) or space (ending Hebrew)
   10 IF(K.EQ.L) GO TO 11
      K=K+1
      IF(HSTR(K).EQ.' ') GO TO 11
      IF(HSTR(K).EQ.'\') GO TO 12
      RC=11
      RETURN
C
C     end of Hebrew; shift the nonzero hash codes left
   11 IF(MC.GT.1) THEN
C        copy the codes left to remove leading zeros
         DO 13 K=MC,MXHASH
               CHASH(K-MC+1)=CHASH(K)
               VHASH(K-MC+1)=VHASH(K)
   13    CONTINUE
C
C        zero out the end codes that were shifted
         DO 14 K=MXHASH-MC+2,MXHASH
               CHASH(K)=0
               VHASH(K)=0
   14    CONTINUE
      ENDIF
      NCH=MXHASH-MC+1
      RC=0
      RETURN
      END
