C
Code by Michael Kupferschmid
C     
      FUNCTION HEBWID(CHASH,VHASH,NC,PT)
C     This routine returns the REAL*4 width in points of the
C     Hebrew word specified by the letter hash codes in CHASH
C     and VHASH.
C
C     variable  meaning
C     --------  -------
C     CHASH     the hash codes of the "consonants"
C     DFLOAT    Fortran function returns REAL*8 for INTEGER*4
C     FLOAT     Fortran function returns REAL*4 for INTEGER*4
C     HEM       the width in points of a shin x 50
C     I         index of a "consonant" specified in CHASH
C     IROUND    function returns REAL*8 rounded to nearest INTEGER*4
C     J         index of a "vowel" specified in VHASH
C     K         index on the characters in VHASH and CHASH
C     L         15ths of a HEM that \hebspc spaces left
C     NC        number of "consonants" specified by CHASH
C     NLTR      number of "consonants" in the table of widths
C     NVWL      number of true vowels defined in hb2hsh.f
C     PT        actual point size of the type whose width is wanted
C     R         15ths of a HEM that \hebspc spaces right
C     VHASH     the hash codes of the corresponding "vowels"
C     WIDTH     widths of the Hebrew characters in hb2hsh.f
C     WSPC      width of a \hebspc space in points x 50
C     WTOT      total width of the specified Hebrew string in points
C     XYLOAD    routine extracts coordinates from a \hebspc hash
C
C     formal parameters
      INTEGER*1 NC,CHASH(NC),VHASH(NC)
      REAL*4 HEBWID,PT
C
C     the consonants listed in hb2hsh.f in hash order
C     aleph    bet      vet      gimel    dalet    hay      vav
C     ohvav    oovav    zayin    khet     tet      yod      endcoph
C     coph     endcaf   caf      lamed    endmem   mem      endnun
C     nun      samech   ayin     endfay   fay      endpay   pay
C     endtsade tsade    kuf      resh     shin     sin      sn
C     taf      dash     space    hash     geresh
C     width in points per 50 of each character at 17 points
      PARAMETER(NLTR=40)
      INTEGER*4 WIDTH(NLTR)/
     ;647,     577,     577,     456,     548,     548,     243,
     ;243,     243,     365,     548,     638,     306,     396,
     ;517,     396,     517,     425,     577,     638,     243,
     ;335,     577,     670,     548,     548,     548,     548,
     ;519,     577,     548,     423,     793,     793,     793,
     ;638,     631,     555,     517,     259/
      INTEGER*4 HEM
      EQUIVALENCE(HEM,WIDTH(33))
C
C     other local variables
      INTEGER*4 NVWL/14/,R,WSPC,WTOT
C
C ------------------------------------------------------------------
C
      WTOT=0
      DO 1 K=1,NC
           J=VHASH(K)
C
C          is there a "consonant" in this place?
           IF(J.LE.NVWL) GO TO 2
C
C          no; does it specify a (zero-width) dagesh or meteg?
           IF(J.EQ.15 .OR. J.EQ.16) GO TO 1
C
C          no; decode the space this \hebspc{L}{R} "vowel" specifies
           CALL XYLOAD(CHASH(K), L,R)
           WSPC=IROUND(DFLOAT((R-L)*HEM)/15.D0)
C
C          add it to the total width accumulated so far
           WTOT=WTOT+WSPC
           GO TO 1
C
C          there is a "consonant" in this space; look up its width
    2      I=CHASH(K)
           IF(I.LT.1 .OR. I.GT.NLTR) THEN
              HEBWID=-FLOAT(K)
              RETURN
           ELSE
C             add its width to that of the string
              WTOT=WTOT+WIDTH(I)
           ENDIF
    1 CONTINUE
C
C     scale by the point size
      HEBWID=(PT/17.0)*(FLOAT(WTOT)/50.0)
      RETURN
      END
