C
Code by Michael Kupferschmid
C
      SUBROUTINE HSH2HB(MXHASH,CHASH,VHASH,MXHSTR, K,HSTR,RC)
C     This routine finds the Hebrew string corresponding to a hash.
C
C     variable  meaning
C     --------  -------
C     CHASH     column of codes for letters in Hebrew string
C     HSTR      Hebrew string to be constructed from hash
C     INS       a name to insert, as a word
C     INSDIG    routine inserts a coordinate like this {x}
C     INSERT    a name to insert, as separate characters
C     K         index on the characters of HSTR
C     KI        index on the characters of an insert
C     L         length of a string
C     LENGTH    function returns index of last nonblank in a string
C     LTR       recognized letter names
C     M         index on hash codes
C     MXHASH    dimensioned length of CHASH and VHASH
C     MXHSTR    dimensioned length of HSTR
C     NLTR      number of letter names
C     NTRP      number of trope names
C     NVWL      number of vowel names
C     RCI       return code from INSDIG
C     RC        return code; 0 or index in hash of error
C     STRBNK    routine blanks out a character string
C     TRP       recognized trope names
C     VHASH     column of codes for vowels in Hebrew string
C     VWL       recognized vowel names
C     X         x-coordinate at which to place a trope mark
C     XYLOAD    routine extracts coordinates from hash
C     Y         y-coordinate at which to place a trope mark
C
C     formal parameters
      CHARACTER*1 HSTR(MXHSTR)
      INTEGER*1 CHASH(MXHASH),VHASH(MXHASH)
      INTEGER*4 RC
C
C     internal variables
      CHARACTER*10 INS
      CHARACTER*1 INSERT(10)
      EQUIVALENCE(INS,INSERT)
      INTEGER*4 X,Y,RCI
C
C     recognized vowel names
      PARAMETER(NVWL=14)
      CHARACTER*10 VWL(NVWL)/'hebrew    ',
     ;                       'chiriq    ',
     ;                       'tsere     ',
     ;                       'segol     ',
     ;                       'halfsegol ',
     ;                       'patach    ',
     ;                       'halfpatach',
     ;                       'qamats    ',
     ;                       'halfqamats',
     ;                       'awe       ',
     ;                       'qubbuts   ',
     ;                       'sheva     ',
     ;                       'lcholam   ',
     ;                       'rcholam   '/
C
C     recognized trope names
      PARAMETER(NTRP=3)
      CHARACTER*10 TRP(NTRP)/'dagesh    ',
     ;                       'meteg     ',
     ;                       'hebspc    '/
C
C     recognized letter names
      PARAMETER(NLTR=40)
      CHARACTER*10 LTR(NLTR)/'aleph     ',
     ;                       'bet       ',
     ;                       'vet       ',
     ;                       'gimel     ',
     ;                       'dalet     ',
     ;                       'hay       ',
     ;                       'vav       ',
     ;                       'ohvav     ',
     ;                       'oovav     ',
     ;                       'zayin     ',
     ;                       'khet      ',
     ;                       'tet       ',
     ;                       'yod       ',
     ;                       'endcoph   ',
     ;                       'coph      ',
     ;                       'endcaf    ',
     ;                       'caf       ',
     ;                       'lamed     ',
     ;                       'endmem    ',
     ;                       'mem       ',
     ;                       'endnun    ',
     ;                       'nun       ',
     ;                       'samech    ',
     ;                       'ayin      ',
     ;                       'endfay    ',
     ;                       'fay       ',
     ;                       'endpay    ',
     ;                       'pay       ',
     ;                       'endtsade  ',
     ;                       'tsade     ',
     ;                       'kuf       ',
     ;                       'resh      ',
     ;                       'shin      ',
     ;                       'sin       ',
     ;                       'sn        ',
     ;                       'taf       ',
     ;                       'dash      ',
     ;                       'space     ',
     ;                       'hash      ',
     ;                       'geresh    '/
C
C ------------------------------------------------------------------
C
      CALL STRBNK(HSTR,MXHSTR)
      K=0
      DO 1 M=MXHASH,1,-1
C          prepare for an error to be detected
           RC=M
C
C          skip zero hash codes; no letter
           IF(VHASH(M).EQ.0) GO TO 1
C
C          check that the vowel-or-trope index is in range
           IF(VHASH(M).LT.1 .OR. VHASH(M).GT.NVWL+NTRP) RETURN
C
C          which does the index designate?
           IF(VHASH(M).LE.NVWL) THEN
C             found a vowel; expand the name
              K=K+1 
              HSTR(K)='\'
              INS=VWL(VHASH(M))
              L=LENGTH(INSERT,10)
              DO 2 KI=1,L
                   K=K+1
                   HSTR(K)=INSERT(KI)
    2         CONTINUE
C
C             check that the letter index is in range
              IF(CHASH(M).LT.1 .OR. CHASH(M).GT.NLTR) RETURN
C
C             expand the letter name
              K=K+1
              HSTR(K)='{'
              INS=LTR(CHASH(M))
              L=LENGTH(INSERT,10)
              DO 3 KI=1,L
                   K=K+1
                   HSTR(K)=INSERT(KI)
    3         CONTINUE
              K=K+1
              HSTR(K)='}'
           ELSE
C             found a trope designation; expand the name
              K=K+1 
              HSTR(K)='\'
              INS=TRP(VHASH(M)-NVWL)
              L=LENGTH(INSERT,10)
              DO 4 KI=1,L
                   K=K+1
                   HSTR(K)=INSERT(KI)
    4         CONTINUE
C
C             extract the coordinates from the hash
              CALL XYLOAD(CHASH(M), X,Y)
C
C             expand the coordinates into the string
              CALL INSDIG(MXHSTR,X, HSTR,K, RCI)
              IF(RCI.GT.0) RETURN              
              CALL INSDIG(MXHSTR,Y, HSTR,K, RCI)
              IF(RCI.GT.0) RETURN              
           ENDIF
    1 CONTINUE
      RC=0
      RETURN
      END
C
      SUBROUTINE INSDIG(MXHSTR,N, HSTR,K, RCI)
C     This routine inserts integer N in a coordinate specification.
C
C     variable  meaning
C     --------  -------
C     BTD       routine converts an INTEGER*4 to numerals
C     HSTR      the Hebrew string under construction
C     K         index in HSTR where next character goes
C     L         length of HSTR
C     N         coordinate to insert
C     RCI       return code; 0 => all went well
C
      CHARACTER*1 HSTR(MXHSTR)
      INTEGER*4 RCI
C
C ------------------------------------------------------------------
C
C     check that the coordinate is in range
      IF(N.LT.0 .OR. N.GT.15) THEN
         RCI=1
         RETURN
      ENDIF
C     
C     expand it within braces
      K=K+1
      HSTR(K)='{'
      K=K+1
      IF(N.LE.9) THEN
         CALL BTD(N, HSTR(K),1,RCI)
         IF(RCI.GT.0) RETURN
         K=K+1
      ELSE
         CALL BTD(N, HSTR(K),2,RCI)
         IF(RCI.GT.0) RETURN
         K=K+2
      ENDIF
      HSTR(K)='}'
      RETURN
      END
