C
Code by Michael Kupferschmid
C
      SUBROUTINE HTPWID(TOKEN,LTK,PT, SUBS,NST,WTK,RC)
C     This routine parses TOKEN into Hebrew, transliterations, and
C     punctuation and adds up their widths.
C
C     RC  meaning
C     --  -------
C     -1  too many subtokens in TOKEN
C     -2  no closing > on a transliteration
C     -3  transliteration not found in the dictionary
C     -4  no closing } on a \vowel{letter} string
C     -5  HB2HSH returned a hash of more than 1 character
C     -6  STRWID did not find a character in its tables
C     -7  HEBWID did not find a hash code in its table
C      0  width successfully estimated
C     >0  return code from HB2HSH
C
C     variable  meaning
C     --------  -------
C     CHASH     the hash code of a Hebrew character
C     DEBUG     T => write debugging output
C     FINDST    routine finds starting location of a substring
C     FIRST     T => this is the first call
C     GETHSH    routine tables lengths and hashcodes of Hebrew words
C     HB2HSH    routine hashes a Hebrew string
C     HEBWID    function gives printed width of Hebrew string in pt
C     I         line number of a transliteration in the dictionary
C     K         index on the characters of TOKEN
C     KERR      unused; from HB2HSH
C     KS        index on the characters of a subtoken
C     KSTART    index in TOKEN where \hebpnk{ starts
C     LS        length of the current subtoken
C     LTK       number of characters in TOKEN
C     LTRS      each column hash codes letters of a dictionary word
C     MXCH      maximum Hebrew characters in a dictionary word
C     MXWD      maximum words in the dictionary
C     NCH       Hebrew characters in a hash string
C     NS        lengths of Hebrew words in the dictionary
C     NST       number of subtokens found in TOKEN
C     NWD       number of dictionary words found by GETHSH
C     PT        pt size at which printed width is to be estimated
C     RC        return code; see table above
C     STRBNK    routine blanks out a character string
C     STRWID    function returns printed width of an English string
C     SUBS      subtokens found in TOKEN
C     TOKEN     the concatenation to be parsed
C     TRAN      a transliteration extracted from TOKEN
C     TRANC     TRAN as separate characters
C     TRANS     list of transliterations in the dictionary
C     TRNIDX    function gives index of <transliteration> in a list
C     VHASH     the hash code of a Hebrew vowel
C     VWLS      each column hash codes vowels in a dictionary word
C     WS        width of a string in pt
C     WTK       predicted print width of TOKEN in pt
C
C     formal parameters
      CHARACTER*1 TOKEN(LTK)
      CHARACTER*1 SUBS(20,*)
      INTEGER*4 RC
      REAL*4 PT
C
C     prepare to read the hashed dictionary only once
      LOGICAL*4 FIRST/.TRUE./
C
C     prepare to store the hashed dictionary, each word in a column
      PARAMETER(MXCH=12,MXWD=6000)
      INTEGER*1 NS(MXWD),LTRS(MXCH,MXWD),VWLS(MXCH,MXWD)
      CHARACTER*18 TRANS(MXWD)
      INTEGER*4 TRNIDX
C
C     prepare to hash a <transliteration>
      CHARACTER*1 TRANC(18)
      CHARACTER*18 TRAN
      EQUIVALENCE(TRAN,TRANC)
      INTEGER*1 CHASH,VHASH
C
C     prepare to write debugging output
      LOGICAL*4 DEBUG/.FALSE./
C
C ------------------------------------------------------------------
C
C     on the first call only, read the hashed dictionary
      IF(FIRST) THEN
         CALL GETHSH(MXCH,MXWD, NWD,NS,TRANS,VWLS,LTRS)
         FIRST=.FALSE.
         IF(DEBUG) WRITE(0,901)
  901    FORMAT('dictionary was read')
      ENDIF
C
C     no subtokens have yet been found
      CALL STRBNK(SUBS(1,1),200)
      WTK=0.0
      K=1
      NST=0
C
C     prepare to store the next subtoken
    7 IF(NST.EQ.10) THEN
         RC=-1
         RETURN
      ENDIF
      NST=NST+1
C
C     does this element of TOKEN begin a <transliteration>?
      IF(TOKEN(K).EQ.'<') THEN
C        yes; copy out the transliteration
         CALL STRBNK(TRANC,18)
         SUBS(1,NST)='<'
         LS=1
         DO 1 KS=K+1,LTK
              IF(TOKEN(KS).EQ.'>') GO TO 2
              TRANC(LS)=TOKEN(KS)
              LS=LS+1
              SUBS(LS,NST)=TOKEN(KS)
    1    CONTINUE
         RC=-2
         RETURN
    2    LS=LS+1
         SUBS(LS,NST)='>'
         IF(DEBUG) WRITE(0,902) NST,(SUBS(KS,NST),KS=1,LS)
  902    FORMAT('subtoken ',I2,' is ',20A1)
C
C        find its line number in the dictionary
         I=TRNIDX(TRAN,TRANS,NWD)
         IF(I.EQ.0) THEN
            RC=-3
            RETURN
         ENDIF
C
C        find the width of the printed Hebrew
         WS=HEBWID(LTRS(1,I),VWLS(1,I),NS(I),PT)
         IF(WS.LT.0) THEN
            RC=-7
            RETURN
         ENDIF
         WTK=WTK+WS
         GO TO 3
      ENDIF
C
C     does this element of TOKEN begin a Hebrew puncutation mark?
      CALL FINDST(TOKEN(K),LTK-K+1,'\hebpnk{',8, KSTART)
      IF(KSTART.EQ.1) THEN
         LS=0
         DO 4 KS=K,K+9
              LS=LS+1
              SUBS(LS,NST)=TOKEN(KS)
    4    CONTINUE
         IF(DEBUG) WRITE(0,902) NST,(SUBS(KS,NST),KS=1,LS)
C
C        add its printed width to the width for the token
         WS=STRWID(SUBS(9,NST),1,PT)
         IF(WS.LT.0) THEN
            RC=-6
            RETURN
         ENDIF
         WTK=WTK+WS
         GO TO 3
      ENDIF
C
C     does this element of TOKEN begin a Hebrew \vowel{consonant}?
      IF(TOKEN(K).EQ.'\') THEN
C        yes; copy it out
         LS=0
         DO 5 KS=K,LTK
              LS=LS+1
              IF(LS.GT.20) THEN
                 RC=-4
                 RETURN
              ENDIF
              SUBS(LS,NST)=TOKEN(KS)
              IF(TOKEN(KS).EQ.'}') GO TO 6
    5    CONTINUE
         RC=-4
         RETURN
    6    IF(DEBUG) WRITE(0,902) NST,(SUBS(KS,NST),KS=1,LS)
C
C        hash the Hebrew
         CALL HB2HSH(20,SUBS(1,NST),1, NCH,CHASH,VHASH,RC,KERR)
         IF(RC.NE.0) RETURN
         IF(NCH.NE.1) THEN
            RC=-5
            RETURN
         ENDIF
C
C        add its printed width to the width for the token
         WS=HEBWID(CHASH,VHASH,NCH,PT)
         IF(WS.LT.0) THEN
            RC=-7
            RETURN
         ENDIF
         WTK=WTK+WS
         GO TO 3
      ENDIF

C     assume this character of TOKEN is English punctuation
         IF(DEBUG) WRITE(0,902) NST,(SUBS(KS,NST),KS=1,LS)
         LS=0
         DO 8 KS=K,LTK
C             copy characters up to the start of the next token
              IF(TOKEN(KS).EQ.'\' .OR. TOKEN(KS).EQ.'<') GO TO 9
              LS=LS+1
              SUBS(LS,NST)=TOKEN(KS)
    8    CONTINUE
C
C        add its printed width to the width for the token
    9    WS=STRWID(SUBS(1,NST),LS,PT)
         IF(WS.LT.0) THEN
            RC=-6
            RETURN
         ENDIF
         WTK=WTK+WS
C
C     move the character pointer in TOKEN past this subtoken
    3 K=K+LS
      IF(K.LE.LTK) GO TO 7
C
C     the end of the token has been reached
      RC=0
      RETURN
      END
