C
Code by Michael Kupferschmid
C
      SUBROUTINE INTINS(STRING,LS,INT,LOC,JOB, RESULT,LR)
C     This routine inserts the numerals corresponding to the
C     integer INT in STRING, starting at location LOC.
C
C     JOB  spacing
C     ---  -------
C      0   leave spacing unchanged
C      1   leave 1 blank at the beginning of the inserted number
C      2   leave no blanks at the beginning of the inserted number
C      3   provide leading sign and zeros
C
C     variable  meaning
C     --------  -------
C     BTD       routine converts an integer to numerals
C     INT       the integer to be inserted
C     JOB       what to do about spacing; see table above
C     K         index on the characters of RESULT
C     KEND      last character to zero out
C     LENGTH    function returns index of last nonblank in a string
C     LINT      number of character positions for the integer
C     LOC       location in STRING where INT is to be inserted
C     LR        index of the last nonblank in RESULT
C     LS        number of characters in STRING
C     MIN0      Fortran function returns smaller of INTEGER*4s
C     RC        return code from BTD; nonzero => error
C     RESULT    result string
C     SHIFTL    routine removes leading blanks from a string
C     STRING    the character string in which INT is to be inserted
C
      CHARACTER*1 STRING(LS),RESULT(*)
      INTEGER*4 RC
C
C ------------------------------------------------------------------
C
C     can't insert into an empty string
      IF(LS.LE.0) THEN
         LR=0
         RETURN
      ENDIF
C
C     copy the template into the result
      DO 1 K=1,LS
           RESULT(K)=STRING(K)
    1 CONTINUE
      LR=LS
C
C     can't insert outside the string
      IF(LOC.LE.0 .OR. LOC.GT.LS) RETURN
C
C     insert the integer, if it will fit
      LINT=MIN0(11,LS-LOC+1)
      CALL BTD(INT, RESULT(LOC),LINT,RC)
C
C     if the integer wouldn't fit, put stars
      IF(RC.NE.0) THEN
         DO 2 K=LOC,LOC+LINT-1
              RESULT(K)='*'
    2    CONTINUE
      ENDIF
C
C     do the requested spacing
      IF(JOB.EQ.1 .AND. LOC.LT.LS) CALL SHIFTL(RESULT(LOC+1),LS-LOC)
      IF(JOB.EQ.2 .AND. LOC.LE.LS) CALL SHIFTL(RESULT(LOC),LS-LOC+1)
      IF(JOB.EQ.3) THEN
         DO 3 K=LOC+LINT-1,LOC,-1
              KEND=K
              IF(RESULT(K).EQ.'-') THEN
                 RESULT(LOC)='-'
                 GO TO 4
              ENDIF
              IF(RESULT(K).EQ.' ') THEN
                 RESULT(LOC)='+'
                 GO TO 4
              ENDIF
    3    CONTINUE
    4    IF(KEND.EQ.LOC) GO TO 5
         DO 6 K=LOC+1,KEND
              RESULT(K)='0'
    6    CONTINUE
      ENDIF
    5 LR=LENGTH(RESULT,LS)
      RETURN
      END
