C
Code by Michael Kupferschmid
C
      SUBROUTINE L2REGW(X,Y,W,M, BETA,RC)
C     This routine does weighted simple linear least-squares
C     regression to find BETA(1) and BETA(2) in
C
C     y(x)=BETA(1)+BETA(2)*x
C
C     so as to minimize the sum of the weighted squared deviations
C     |W(I)|*(y(X(I))-Y(I))^2  i=1...m.
C
C     RC  meaning
C     --  -------
C     -1  parameter error
C      0  all went well
C      1  the regression line is vertical
C
C     variable  quantity
C     --------  --------
C     BETA      vector of regression coefficients (intercept,slope)
C     DABS      Fortran function returns |REAL*8|
C     DB1       BETA(1)
C     DB2       BETA(2)
C     DW        W(I)
C     DX        X(I)
C     DY        Y(I)
C     I         index on the elements of W, X, and Y
C     M         number of data points
C     RC        return code; see table above
C     SW        sum of the W(I)s
C     SXW       sum of X(I)*W(I) products
C     SXXW      sum of X(I)*X(I)*W(I) products
C     SXYW      sum of X(I)*Y(I)*W(I) products
C     SYW       sum of Y(I)*W(I) products
C     W         vector of weights
C     X         vector of independent variable values
C     Y         vector of corresponding dependent variable values
C
C     formal parameters
      REAL*8 X(M),Y(M),W(M),BETA(2)
      INTEGER*4 RC
C
C     local variables
      REAL*8 DB1,DB2,DW,DX,DY,SW,SXW,SXXW,SXYW,SYW
C
C ------------------------------------------------------------------
C
C     are there any data points?
      BETA(1)=0.D0
      BETA(2)=0.D0
      IF(M.LE.0) THEN
         RC=-1
         RETURN
      ENDIF
C
C     is there only one data point?
      IF(M.EQ.1) THEN
         BETA(1)=Y(1)
         RC=0
         RETURN
      ENDIF
C
C     more than one data point; compute various sums
      SW=0.D0
      SXYW=0.D0
      SYW=0.D0
      SXXW=0.D0
      SXW=0.D0
      DO 1 I=1,M
           DX=X(I)
           DY=Y(I)
           DW=DABS(W(I))
C
           SW=SW+DW
           SXYW=SXYW+DX*DY*DW
           SYW=SYW+DY*DW
           SXXW=SXXW+DX*DX*DW
           SXW=SXW+DX*DW
    1 CONTINUE
C
C     are there any nonzero weights?
      IF(SW.EQ.0.D0) THEN
         RC=-1
         RETURN
      ENDIF
      IF(SXXW-SXW*SXW/SW .EQ. 0.D0) THEN
         RC=1
         RETURN
      ENDIF
C
C     compute the least-squares fit
      DB2=(SXYW-SXW*SYW/SW)/(SXXW-SXW*SXW/SW)
      DB1=(SYW-DB2*SXW)/SW
C
C     return the regression coefficients
      BETA(1)=DB1
      BETA(2)=DB2
      RC=0
      RETURN
      END
