C
Code by Michael Kupferschmid
C
      SUBROUTINE LEXHEB(MXCH,NWD,LS,VS, ORDER)
C     This routine sorts the INTEGER*1 vectors in LS|VS
C     into ascending lexicographical order.
C
C     variable  meaning
C     --------  -------
C     CS        for each word, letters followed by vowels
C     I         index on words (columns of CS)
C     J         second index on words for sorting
C     K         index on characters [letters,vowels] in a column
C     L         index on rows for exchanging columns of LS
C     LS        letter hash codes
C     MXCH      dimensioned row size of LS and VS
C     NWD       number of words (columns) to sort
C     ORDER     starting order of columns in LS, then ending
C     TEMP1     temporary for INTEGER*1 exchanges
C     TEMP4     temporary for INTEGER*4 exchanges
C     VS        vowel hash codes
C
C     formal parameters
      INTEGER*1 LS(MXCH,NWD),VS(MXCH,NWD)
      INTEGER*4 ORDER(NWD)
C
C     local and automatic variables 
      INTEGER*1 TEMP1
      INTEGER*4 TEMP4
      INTEGER*1 CS(2*MXCH,NWD)
C
C ------------------------------------------------------------------
C
C     copy the vowel hash codes after the letter hash codes
      DO 1 I=1,NWD
           DO 2 K=1,MXCH
                CS(K,I)=LS(K,I)
    2      CONTINUE
           DO 3 K=1,MXCH
                CS(MXCH+K,I)=VS(K,I)
    3      CONTINUE
    1 CONTINUE
C
C     bubble sort of columns
      DO 4 I=1,NWD-1
      DO 4 J=I+1,NWD
C          lexicographical comparison of column I with column J
           DO 5 K=1,2*MXCH
                IF(CS(K,I).EQ.CS(K,J)) GO TO 5
C               columns equal so far => keep comparing down column
C
                IF(CS(K,I).LT.CS(K,J)) GO TO 4
C               correct order => don't swap columns
C
                GO TO 6
C               wrong order => stop comparing and swap
    5      CONTINUE
C          columns equal => don't swap them
           GO TO 4
C
C          step down rows (characters) swapping columns (words)
    6      DO 7 L=1,2*MXCH
                TEMP1=CS(L,I)
                CS(L,I)=CS(L,J)
                CS(L,J)=TEMP1
    7      CONTINUE
C
C          remember the new order of the columns
           TEMP4=ORDER(I)
           ORDER(I)=ORDER(J)
           ORDER(J)=TEMP4
    4 CONTINUE
C
C     report duplicates
      DO 8 I=1,NWD-1
           DO 9 K=1,2*MXCH
                IF(CS(K,I).NE.CS(K,I+1)) GO TO 8
    9      CONTINUE
           II=ORDER(I)
           WRITE(0,901) (VS(K,II),LS(K,II),K=1,MXCH)
  901      FORMAT(22I2.2,' character sequence repeats')
    8 CONTINUE
      RETURN
      END
