C
Code by Michael Kupferschmid
C
      SUBROUTINE LSTMOD(FILE,LF, WHEN,RC)
C     This routine returns the last modification time of a file.
C     It makes transient use of logical I/O unit 99.
C
C     RC  meaning
C     --  -------
C      0  all went well
C      1  FILE is too long
C      2  the /bin/ls command failed
C      3  read of result failed
C
C     variable  meaning
C     --------  -------
C     CMD       /bin/ls command as one long word
C     CMDC      /bin/ls --full-time filename > /tmp/=time
C     DATE      standard-form date of last modification
C     DAY       day of the week of last modification
C     DAYCH     DAY as separate characters
C     DAYNO     day number of DATE
C     DAYOF     function returns day number of given short-form date
C     DY        day number in month of last modification
C     FILE      name of file whose last modification date is sought
C     HR        hour of day of last modification
C     K         index on the characters of a string
C     LENGTH    function returns index of last nonblank in a string
C     LF        length of FILE
C     LR        length of result string from ls -l command
C     MIN       minute of day of last modification
C     MO        integer month of year of last modification
C     MON       character month of year of last modification
C     MONCH     MON a sseparate characters
C     MONTHS    the names of the months
C     NF        number of result field just blanked out
C     RC        return code; 0 => all went well
C     RESULT    result string produced by ls -l command
C     RESWRD    RESULT as one long word for rereading
C     SEC       second of day of last modification
C     SHIFTL    routine removes leading blanks from a string
C     SPACEL    routine removes leading and extra blanks
C     SYSTEM    routine issues a unix command
C     TMPLT     command template with empty filename
C     WHEN      day, date, and time of last file modification
C     WKDAY     routine returns day of the week given a day number
C     YR        year of last modification
C
C     formal parameters
      CHARACTER*1 FILE(LF)
      CHARACTER*1 WHEN(22)
      INTEGER*4 RC
C
C     prepare to issue the ls command
      CHARACTER*1 TMPLT(70)/'/','b','i','n','/','l','s',' ','-','-',
     ;                      'f','u','l','l','-','t','i','m','e',' ',
     ;                      ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
     ;                      ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
     ;                      ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
     ;                      ' ',' ',' ',' ',' ',' ',' ',' ','>',' ',
     ;                      '/','t','m','p','/','=','t','i','m','e'/
      CHARACTER*1 CMDC(70)/70*' '/
      CHARACTER*70 CMD
      EQUIVALENCE(CMDC,CMD)

C     prepare to parse the output from /bin/ls
      CHARACTER*1 RESULT(128)
      CHARACTER*128 RESWRD
      EQUIVALENCE(RESULT,RESWRD)
      CHARACTER*1 YR(2),DY(2),HR(2),MIN(2),SEC(2)
C
C     prepare to construct the modification day-date-time string
      CHARACTER*3 MONTHS(12)/'Jan','Feb','Mar','Apr','May','Jun',
     ;                       'Jul','Aug','Sep','Oct','Nov','Dec'/
      CHARACTER*3 MON
      CHARACTER*1 MONCH(3)
      EQUIVALENCE(MON,MONCH)
      CHARACTER*1 DATE(9)
      INTEGER*4 DAYNO,DAYOF
      CHARACTER*3 WKDAY,DAY
      CHARACTER*1 DAYCH(3)
      EQUIVALENCE(DAYCH,DAY)
C
C ------------------------------------------------------------------
C
C     check that the filename fits in the command template
      DO 1 K=1,22
           WHEN(K)=' '
    1 CONTINUE
      RC=1
      IF(LF.GT.37) RETURN

C     construct the /bin/ls command
      DO 2 K=1,20
           CMDC(K)=TMPLT(K)
    2 CONTINUE
      DO 3 K=21,21+LF-1
           CMDC(K)=FILE(K-20)
    3 CONTINUE
      DO 4 K=21+LF,47
           CMDC(K)=' '
    4 CONTINUE
      DO 5 K=58,70
           CMDC(K)=TMPLT(K)
    5 CONTINUE
      CALL SPACEL(CMDC,70)
C
C     issue the /bin/ls command
      RC=SYSTEM('/bin/rm -f /tmp/=time')
      RC=SYSTEM(CMD)
      IF(RC.GT.0) THEN
         RC=2
         RETURN
      ENDIF
C
C     read the result
      OPEN(UNIT=99,FILE='/tmp/=time',STATUS='OLD')
      READ(99,901,ERR=6) RESULT
  901 FORMAT(128A1)
      CLOSE(UNIT=99)
      LR=LENGTH(RESULT,128)
C
C     blank out the result up to the start of the date field
      NF=1
      DO 7 K=1,LR
           IF(RESULT(K).EQ.' ') NF=NF+1
           IF(NF.EQ.6) GO TO 8
           RESULT(K)=' '
    7 CONTINUE
      GO TO 6
    8 CALL SHIFTL(RESULT,LR)
C
C     extract the date and time the file was last changed
      READ(RESWRD,902) YR,MO,DY,HR,MIN,SEC
  902 FORMAT(2X,2A1,1X,I2,1X,2A1,1X,2A1,1X,2A1,1X,2A1)
      DATE(1)=DY(1)
      DATE(2)=DY(2)
      DATE(3)=' '
      MON=MONTHS(MO)
      DATE(4)=MONCH(1)
      DATE(5)=MONCH(2)
      DATE(6)=MONCH(3)
      DATE(7)=' '
      DATE(8)=YR(1)
      DATE(9)=YR(2)
C
C     construct the time-date string
C       weekday name
        DAYNO=DAYOF(DATE)
        DAY=WKDAY(DAYNO)
        WHEN(1)=DAYCH(1)
        WHEN(2)=DAYCH(2)
        WHEN(3)=DAYCH(3)

        WHEN(4)=' '
C       standard-format date
        DO 9 K=1,9
             WHEN(4+K)=DATE(K)
    9   CONTINUE
        WHEN(14)=' '
C
C       time of day
        WHEN(15)=HR(1)
        WHEN(16)=HR(2)
        WHEN(17)=':'
        WHEN(18)=MIN(1)
        WHEN(19)=MIN(2)
        WHEN(20)=':'
        WHEN(21)=SEC(1)
        WHEN(22)=SEC(2)
C
C     signal success
      RC=0
      RETURN
C
C     the read of /tmp/=time failed
    6 RC=3
      RETURN
      END
