C
Code by Michael Kupferschmid
C
      SUBROUTINE MINR(T,LDT,ROWS,MR,JP, IP)
C     This routine finds the min ratio row IP in column JP of T.
C
C     variable  meaning
C     --------  -------
C     EPS       pivot tolerance
C     HUGE      machine infinity
C     I         index on the rows listed in ROWS
C     IP        min ratio pivot row, or 0 if no entry > 0 in col JP
C     JP        pivot column in which we are to search for min ratio
C     LDT       leading dimension of T
C     MR        number of rows in the problem (could be a subprob)
C     R         ratio for this row
C     RMIN      smallest value of R
C     ROWS      vector of row indices in the problem
C     T         the tableau
C
C     formal parameters
      REAL*8 T(LDT,*)
      INTEGER*4 ROWS(MR)
C
C     local variables
      REAL*8 R,RMIN,EPS/1.D-6/,HUGE/1.D+100/
C
C ------------------------------------------------------------------
C
C     check for a legal value of JP
      IF(JP.LE.1) THEN
         WRITE(6,901) JP
  901    FORMAT('Pivot column ',I11,' is not a variable column.')
         STOP
      ENDIF
C
C     look down the column and find the row having the minimum ratio
      IP=0
      RMIN=HUGE
      DO 1 I=2,MR
           IF(T(ROWS(I),JP) .LE. EPS) GO TO 1
           R=T(ROWS(I),1)/T(ROWS(I),JP)
           IF(R.LT.RMIN) THEN
              RMIN=R
              IP=ROWS(I)
           ENDIF
    1 CONTINUE
      RETURN
      END
