C
Code by Michael Kupferschmid
C
      SUBROUTINE MPYACC(X,Y, XYSUM )
C     This routine accumulates XYSUM=XYSUM+X*Y at extra precision.
C
C     variable  meaning
C     --------  -------
C     ADDACC    routine adds to an extra-precision accumulator
C     HMASK     deletes the 26 least-significant fraction bits
C     IAND      Fortran function for bitwise logical AND
C     IXH       XH as 2 fullwords
C     IYH       YH as 2 fullwords
C     P         a partial product
C     X         first number in product
C     XH        split of X containing its high 26 fraction bits
C     XT        split of X containing value of trailing 26 bits
C     XYSUM     extra-precision accumulator
C     Y         second number in product
C     YH        split of Y containing its high 26 fraction bits
C     YT        split of Y containing value of trailing 26 bits
C
C     formal parameters
      REAL*8 X,Y,XYSUM(2)
C
C     prepare to split X and Y
      REAL*8 XH,XT,YH,YT
      INTEGER*4 IXH(2),IYH(2)
      EQUIVALENCE(XH,IXH),(YH,IYH)
      INTEGER*4 HMASK/Z'FC000000'/
C
C     prepare to compute the partial products
      REAL*8 P
C
C ------------------------------------------------------------------
C
C     split X and Y into parts having 26 trailing fraction bits zero
      XH=X
C     this assumes the processor is little-endian
      IXH(1)=IAND(IXH(1),HMASK)
      XT=X-XH
      YH=Y
C     this assumes the processor is little-endian
      IYH(1)=IAND(IYH(1),HMASK)
      YT=Y-YH
C
C     add the 52-fraction-bit exact partial products to accumulator
      P=XT*YT
      CALL ADDACC(P,XYSUM)
      P=XT*YH
      CALL ADDACC(P,XYSUM)
      P=XH*YT
      CALL ADDACC(P,XYSUM)
      P=XH*YH
      CALL ADDACC(P,XYSUM)
      RETURN
      END
