C
Code by Michael Kupferschmid
C
      SUBROUTINE PHASE0(T,LDT,BASIS,MP,N,ROWS,MR, RCZ)
C     This routine gets the identity columns with zero costs above.
C
C     variable  meaning
C     --------  -------
C     BASIS     indices in T of 1s in basic columns, 0 for nonbasic
C     DABS      Fortran function returns |REAL*8|
C     EPS       pivot tolerance
C     I         index on the rows of T
C     II        index on rows of T being copied
C     IIR       second index on ROWS entries
C     IR        index on the entries in ROWS
C     J         index on the columns of T
C     JJ        index on columns of T being copied
C     LDT       leading dimension of T
C     MP        number of rows total
C     MR        number of rows in use
C     N         number of variables in the problem
C     PIVOT     subroutine performs a single pivot
C     RCZ       return code; 0 => ok, 1 => inf form 1
C     ROWS      indices of the MR tableau rows in use
C     T         the tableau
C
C     formal parameters
      REAL*8 T(LDT,*)
      INTEGER*4 ROWS(MR),BASIS(N),RCZ
C
C     local variable
      REAL*8 EPS/1.D-06/
C
C ------------------------------------------------------------------
C
C     assume there is no basis to begin with
      DO 1 J=2,N+1
           BASIS(J-1)=0
    1 CONTINUE
C
C     but assume there will be one when we are finished
      RCZ=0
C
C     the first row in ROWS is the tableau row of the objective
      IR=1
C
C     consider each of the constraint rows
    7 IR=IR+1
    6 I=ROWS(IR)
C       find the first nonzero entry in this row
        DO 2 J=2,N+1
             IF(DABS(T(I,J)).LE.EPS) GO TO 2
C
C            pivot on the nonzero element
             CALL PIVOT(T,LDT,MP,N,I,J, BASIS, T)
             GO TO 4
    2   CONTINUE
C
C       the a_{i,j} are all zero; check the constant column entry
        IF(DABS(T(I,1)).LE.EPS) THEN
C          the row is all zeros; is it the last row in the problem?
           IF(I.LT.MP) THEN
C             it is not the last row; adjust the indices in ROWS
              DO 5 IIR=IR,MR-1
                   ROWS(IIR)=ROWS(IIR+1)
    5         CONTINUE
           ENDIF
C          decrease the number of rows in use
           MR=MR-1
           IF(IR.LT.MR) GO TO 6
           RETURN
        ELSE
C          the problem is in infeasible form 1
           RCZ=1
           RETURN
        ENDIF
    4 IF(IR.LT.MR) GO TO 7
      RETURN
      END
