C
Code by Michael Kupferschmid
C
      SUBROUTINE PHASE1(T,LDT,BASIS,MP,N,ROWS,MR, RC1)
C     This routine gets the constant column nonnegative, or
C     discovers the problem is infeasible.
C
C     variable  meaning
C     --------  -------
C     BASIS     row indices of 1s in basic columns, 0 for nonbasic
C     DABS      Fortran function returns |REAL*8|
C     EPS       zero tolerance for subproblem objective
C     I         index on the rows of the tableau
C     II        second index on rows, for copying the tableau
C     IP        pivot row
C     IR        index on the elements of ROWS
C     J         index on the columns of the tableau
C     JJ        second index on columns, for copying the tableau
C     JP        pivot column
C     LDT       leading dimension of T
C     MP        number of rows total
C     MR        number of rows in use
C     MS        number of rows in subproblem, including subpr obj
C     N         number of variables in the problem
C     PHASE2    routine solves a canonical-form LP
C     PIVOT     routine performs a pivot in T
C     RC1       0 => ready; 2 => infeasible 2
C     RC2       from PHASE2; 0 => optimal; >0 => unbdd in col RC2
C     ROWS      rows of T in use for the main problem
C     SROWS     rows of T in subproblem
C     T         the tableau
C
C     formal parameters
      REAL*8 T(LDT,*)
      INTEGER*4 BASIS(N),ROWS(MR),RC1
C
C     local variables
      INTEGER*4 RC2,SROWS(MR)
      REAL*8 EPS/1.D-06/
C
C ------------------------------------------------------------------
C
C     look for a nonnegative b
      DO 1 IR=2,MR
           I=ROWS(IR)
           IF(T(I,1).GE.0.D0) GO TO 2
    1 CONTINUE
C
C     all the b's are negative; try to make the first one positive
      DO 3 J=2,N+1
           JP=J
           IF(T(ROWS(2),J).LT.0.D0) GO TO 4
    3 CONTINUE
      RC1=2
      RETURN
    4 CALL PIVOT(T,LDT,MP,N,ROWS(2),JP, BASIS, T)
C
C     construct a subproblem, or discover there are none left
    2 MS=1
      SROWS(1)=0
      DO 6 IR=2,MR
           I=ROWS(IR)
           IF(T(I,1).LT.0.D0) THEN
C             row containing objective function for the subproblem
              IF(SROWS(1).EQ.0) SROWS(1)=I
           ELSE
C             rows containing constraints for the subproblem
              MS=MS+1
              SROWS(MS)=I
           ENDIF
    6 CONTINUE
      IF(SROWS(1).EQ.0) THEN
         RC1=0
         RETURN
      ENDIF
C
C     solve the subproblem
      CALL PHASE2(T,LDT,BASIS,MP,N,SROWS,MS, RC2)
      IF(DABS(T(SROWS(1),1)).LT.EPS) T(SROWS(1),1)=0.D0
C
C     optimal subproblem with negative -z means infeasible form 2
      IF(RC2.EQ.0 .AND. T(SROWS(1),1).LT.0.D0) THEN
         RC1=2
         RETURN
      ENDIF
C
C     unbounded subproblem with negative -z means pivot
      IF(RC2.GT.0 .AND. T(SROWS(1),1).LT.0.D0) THEN
         JP=RC2
         IP=SROWS(1)
         CALL PIVOT(T,LDT,MP,N,IP,JP, BASIS, T)
      ENDIF
      GO TO 2
      END
