C
Code by Michael Kupferschmid
C
      SUBROUTINE PHASE2(T,LDT,BASIS,MP,N,ROWS,MR, RC2)
C     This routine optimizes a tableau in canonical form,
C     or determines that it is unbounded.
C
C     variable  meaning
C     --------  -------
C     BASIS     row indices of 1s in basic columns, 0 for nonbasic
C     CMIN      most negative cost found
C     EPS       zero tolerance for cost coefficients
C     I         index on the rows of T
C     IP        pivot row, found by MINR
C     J         index on the columns of T
C     JP        pivot column
C     LDT       leading dimension of T
C     MINR      routine determines the minimum ratio row
C     MP        number of rows in total
C     MR        number of rows in use (could be a subprob)
C     N         number of variables in the problem
C     PIVOT     routine performs a single pivot
C     RC2       0 => optimal; >0 => unbounded in column RC2
C     ROWS      vector of row indices in the problem
C     T         the tableau
C
C     formal parameters
      REAL*8 T(LDT,*)
      INTEGER*4 BASIS(N),ROWS(MR),RC2
C
C     local variable
      REAL*8 CMIN,EPS/1.D-06/
C
C  -----------------------------------------------------------------
C
C     find the most negative cost entry
    3 CMIN=0.D0
      JP=0
C     in the objective row
      I=ROWS(1)
      DO 1 J=2,N+1
           IF(T(I,J).LT.CMIN) THEN
              CMIN=T(I,J)
              JP=J
           ENDIF
    1 CONTINUE
      IF(JP.EQ.0 .OR. CMIN.GT.-EPS) THEN
         RC2=0
         RETURN
      ENDIF
C
C     find the minimum-ratio row in that column
      CALL MINR(T,LDT,ROWS,MR,JP, IP)
      IF(IP.EQ.0) THEN
         RC2=JP
         RETURN
      ENDIF
C
C     perform the pivot
      CALL PIVOT(T,LDT,MP,N,IP,JP, BASIS, T)
      GO TO 3
      END
