C
Code by Michael Kupferschmid
C
      SUBROUTINE PIVOT(TIN,LDT,MP,N,IP,JP, BASIS, TOUT)
C     This routine performs a pivot in TIN at element (IP,JP),
C     producing TOUT.  TOUT can be the same array as TIN.
C
C     variable  meaning
C     --------  -------
C     BASIS     row index in TIN of 1 in basic column, else 0
C     I         index on rows of the tableau
C     IP        pivot row
C     J         index on columns of the tableau
C     JP        pivot column
C     LDT       leading dimension of TIN and TOUT
C     MP        number of rows
C     N         number of variables
C     TIN       the input tableau
C     TOUT      the output tableau
C
C     formal parameters
      REAL*8 TIN(LDT,*),TOUT(LDT,*)
      INTEGER*4 BASIS(N)
C
C ------------------------------------------------------------------
C
C     check that the pivot element is at a place where we can pivot
      IF((IP.LE.1 .OR. IP.GT.MP) .OR.
     ;   (JP.LE.1 .OR. JP.GT.N+1)) THEN
        WRITE(0,901) IP,JP,MP,N+1
  901   FORMAT('in PIVOT, element (',I10,',',I10,
     ;         ') is not in TIN(',I10,',',I10,').')
        STOP
      ENDIF
C
C     check that the pivot element is not zero
      IF(TIN(IP,JP).EQ.0.D0) THEN
         WRITE(6,902) IP,JP
  902    FORMAT('Pivot element (',I3,',',I3,') is zero.')
         STOP
      ENDIF
C
C     compute new elements not in pivot row or pivot column
      DO 1 I=1,MP
           IF(I.EQ.IP) GO TO 1
           DO 2 J=1,N+1
                IF(J.EQ.JP) GO TO 2
                TOUT(I,J)=TIN(I,J)-TIN(I,JP)*TIN(IP,J)/TIN(IP,JP)
    2      CONTINUE
    1 CONTINUE
C
C     compute the new pivot row, except for the pivot element
      DO 3 J=1,N+1
           IF(J.EQ.JP) GO TO 3
           TOUT(IP,J)=TIN(IP,J)/TIN(IP,JP)
    3 CONTINUE
C
C     fill in the new pivot column (including the pivot element)
      DO 4 I=1,MP
           IF(I.EQ.IP) THEN
              TOUT(I,JP)=1.D0
           ELSE
              TOUT(I,JP)=0.D0
           ENDIF
    4 CONTINUE
C
C     the variable having a 1 in the pivot row became nonbasic
      DO 5 J=2,N+1
           IF(BASIS(J-1).EQ.IP) BASIS(J-1)=0
    5 CONTINUE
C
C     the variable corresponding to the pivot column became basic
      BASIS(JP-1)=IP
      RETURN
      END
