C
Code by Michael Kupferschmid
C
      SUBROUTINE PRNHST(NUNIT,TITLE,LT,NOBS,PDF,LPDF,NC,MARGIN, RC)
C     This routine prints the histogram in PDF horizontally.
C
C     RC  meaning
C     --  -------
C      0  all went well
C      1  NUNIT has a bad value
C      2  a histogram parameter has a bad value
C      3  the display margin is too narrow or too wide
C      4  format string construction failed
C
C     variable  meaning
C     --------  -------
C     BARS      numbers of dots to print for the bars
C     BDYMAX    cell boundary largest in absolute value
C     BDYMIN    nonzero cell boundary smallest in absolute value
C     BTD       routine converts INTEGER*4 to numerals
C     DABS      Fortran function returns |REAL*8|
C     DFLOAT    Fortran function returns REAL*8 for INTEGER*4
C     DMAX1     Fortran function returns larger of REAL*8s
C     FMT       object-time format string
C     I         index on the cells of the histogram
C     IROUND    function returns INTEGER*4 round of a REAL*8
C     K         index on the characters in a string
C     LPDF      leading dimension of PDF
C     LT        number of characters in TITLE
C     MARGIN    desired width of display in characters
C     MIN0      Fortran function returns smaller of INTEGER*4s
C     NC        number of cells
C     NOBS      number of observations
C     NUNIT     I/O unit to write the display on
C     PDF       probability density function
C     PDFMAX    largest cell count
C     PDFSUM    sum of cell counts
C     RC        return code; see table above
C     RCB       return code from BTD
C     SCALE     scale factor to convert density to bar length
C     TITLE     description of quantity being histogrammed
C
C     formal parameters
      CHARACTER*1 TITLE(LT)
      REAL*8 PDF(LPDF,2)
      INTEGER*4 RC
C
C     local variables
      INTEGER*4 BARS(NC),RCB
      REAL*8 PDFMAX,PDFSUM,BDYMAX,BDYMIN,SCALE
      CHARACTER*17 FMT
C
C ------------------------------------------------------------------
C
C     sanity-check the input parameters
      RC=0
      IF(NUNIT.LE.0 .OR. NUNIT.GT.99 .OR. NUNIT.EQ.5) RC=1
      IF(NOBS.LE.0 .OR. LPDF.LT.2 .OR. NC.LE.0) RC=2
      IF(MARGIN.LE.10 .OR. MARGIN.GT.1009) RC=3
      IF(RC.NE.0) RETURN
C
C     find out about the data
      PDFMAX=0.D0
      PDFSUM=0.D0
      BDYMAX=-1.D100
      BDYMIN=+1.D100
      DO 1 I=1,NC
           PDFMAX=DMAX1(PDFMAX,PDF(I,2))
           PDFSUM=PDFSUM+PDF(I,2)
           IF(PDF(I,1).EQ.0.D0) GO TO 1
           IF(DABS(PDF(I,1)).GT.BDYMAX) BDYMAX=PDF(I,1)
           IF(DABS(PDF(I,1)).LT.BDYMIN) BDYMIN=PDF(I,1)
    1 CONTINUE
C
C     is it possible to print a dot for each observation?
      SCALE=DFLOAT(NOBS)/PDFSUM
      IF(10+IROUND(SCALE*PDFMAX).GT.MARGIN) THEN
C        no; adjust scaling so the longest bar fits
         SCALE=DFLOAT(MARGIN-10)/PDFMAX 
      ENDIF
C
C     figure out the lengths of the bars
      DO 2 I=1,NC
           BARS(I)=IROUND(SCALE*PDF(I,2))
    2 CONTINUE
C
C     figure out how to format the cell boundary values
      IF(DABS(BDYMIN).GE.0.1D0 .AND. DABS(BDYMAX).LE.999.0D0) THEN
         FMT='(0PF9.2,1X,xxxA1)'
      ELSE
         FMT='(1PE9.2,1X,xxxA1)'
      ENDIF
      CALL BTD(MARGIN-10,FMT(12:14),3,RCB)
      IF(RCB.NE.0) THEN
         RC=4
         RETURN
      ENDIF
C
C     display the histogram
      WRITE(NUNIT,901) 
  901 FORMAT(' ')
      IF(LT.GT.0) WRITE(NUNIT,902) (TITLE(K),K=1,MIN0(LT,79))
  902 FORMAT(' ',79A1)
      DO 3 I=1,NC
           WRITE(NUNIT,FMT) PDF(I,1),('.',K=1,BARS(I))
    3 CONTINUE
      WRITE(NUNIT,FMT) PDF(NC+1,1)
      WRITE(NUNIT,901)
      RETURN
      END
