C
Code by Michael Kupferschmid
C
      SUBROUTINE RNGLST(TOKEN,LT, LIST,LL, NL,RC)
C     This routine expands a range specification like 12-15 and
C     appends the NL integers to LIST.
C
C     RC  meaning
C     --  -------
C      0  all went well
C      1  parse error
C      2  more than LL entries for LIST
C      3  the input parameters did not make sense
C
C     variable  meaning
C     --------  -------
C     DTB       routine converts numerals to an INTEGER*4
C     I         index on integers in the range
C     INC       +1 if counting forward, -1 if counting backward
C     ISTART    first integer in the range
C     ISTOP     second integer in the range
C     J         index on the integers in LIST
C     K         index on the characters in TOKEN
C     LIST      list of integers appended to from given range
C     LL        dimensioned size of LIST
C     LT        number of characters in TOKEN
C     NL        number of integers added to LIST
C     RC        return code; see table above
C     TOKEN     character string containing range specification
C
      CHARACTER*1 TOKEN(LT)
      INTEGER*4 LIST(LL),RC
C
C ------------------------------------------------------------------
C
C     to begin with no integers have been added to LIST
      NL=0
C
C     sanity-check the input parameters
      RC=3
      IF(LT.LT.1 .OR. LL.LT.1) RETURN
C
C     parse out the limits of the range
      DO 1 K=1,LT
           IF(TOKEN(K).NE.'-') GO TO 1
C
C          found the dash; convert the starting value
           CALL DTB(TOKEN,K-1, ISTART,RC)
           IF(RC.NE.0) RETURN
C
C          convert the ending value
           CALL DTB(TOKEN(K+1),LT-K, ISTOP,RC)
           IF(RC.NE.0) RETURN
C
C          done parsing
           GO TO 2
    1 CONTINUE
C
C     no dash found; maybe it is a single number
      CALL DTB(TOKEN,LT, I,RC)
      IF(RC.NE.0) RETURN
      IF(NL.GE.LL) THEN
         RC=2
         RETURN
      ENDIF
      NL=NL+1
      LIST(NL)=I
      RETURN 
C
C     figure out which direction the range goes
    2 IF(ISTOP.GE.ISTART) INC=+1
      IF(ISTOP.LT.ISTART) INC=-1
C
C     enumerate the values in the range
      DO 3 I=ISTART,ISTOP,INC
           IF(NL.GE.LL) THEN
              RC=2
              RETURN
           ENDIF
           NL=NL+1
           LIST(NL)=I
    3 CONTINUE
      RC=0
      RETURN
      END
