C
Code by Michael Kupferschmid
C
      SUBROUTINE SIMPLX(T,LDT,MP,N,ROWS,MR, BASIS,X,RC)
C     This code uses the simplex algorithm to solve a linear
C     programming problem in standard form.
C
C     RC  meaning
C     --  -------
C     -1  parameter error
C      0  all went well; T is in optimal form
C      1  the problem is infeasible
C     >1  the problem is unbounded in tableau column RC
C
C     variable  meaning
C     --------  -------
C     BASIS     row indices of 1s in basic columns, 0 for nonbasic
C     I         index on the rows of the tableau
C     J         index on the variables
C     LDT       leading dimension of T
C     MP        number of rows total
C     MR        number of rows in use
C     N         number of variables
C     PHASE0    routine gets identity columns with zero costs
C     PHASE1    routine gets b nonnegative
C     PHASE2    routine optimizes
C     RC        return code; see table above
C     RCZ       return code; 0 => ok; 1 => inf form 1
C     RC1       return code; 0 => ok, 2 => inf form 2
C     RC2       return code; 0 => ok, >0 => unbounded in column RC2
C     ROWS      indices of the rows in the problem
C     T         the tableau
C     X         solution vector
C
C     formal parameters
C       tableau input and output
        REAL*8 T(LDT,*)
C
C       workspace
        INTEGER*4 ROWS(MR)
C
C       outputs
        REAL*8 X(N)
        INTEGER*4 BASIS(N),RC
C
C     local variables
      INTEGER*4 RCZ,RC1,RC2
C
C ------------------------------------------------------------------
C
C     check for sensible input parameters
      IF(LDT.LE.0 .OR. MP.LE.1 .OR. N.LE.0 .OR. MR.LE.1) THEN
         RC=-1
         RETURN
      ENDIF
C
C     get the identity columns with zero costs above
      CALL PHASE0(T,LDT,BASIS,MP,N,ROWS,MR, RCZ)
      IF(RCZ.NE.0) THEN
         RC=1
         RETURN
      ENDIF
C
C     get b nonnegative by using the subproblem technique
      CALL PHASE1(T,LDT,BASIS,MP,N,ROWS,MR, RC1)
      IF(RC1.NE.0) THEN
         RC=1
         RETURN
      ENDIF
C
C     get optimal form
      CALL PHASE2(T,LDT,BASIS,MP,N,ROWS,MR, RC2)
      RC=RC2
C
C     optimal form; extract the solution vector
      DO 1 J=1,N
           I=BASIS(J)
           IF(I.EQ.0) THEN
C             this variable is nonbasic
              X(J)=0.D0
           ELSE
C             this variable is basic
              X(J)=T(I,1)
           ENDIF
    1 CONTINUE
      RETURN
      END
