C
Code by Michael Kupferschmid
C
      FUNCTION STRSIM(STR1,L1,STR2,L2)
C     This routine scores two strings for similarity.
C
C     variable  meaning
C     --------  -------
C     DFLOAT    Fortran function returns REAL*8 for INTEGER*4
C     K         index on the characters of a string
C     KK        second index on match indices for sorting
C     KL        index on the characters in LONG
C     KS        index on the characters in SHORT
C     L1        length of STR1
C     L2        length of STR2
C     LL        length of LONG
C     LMAX      maximum number of characters allowed in strings
C     LONG      the longer of the input strings
C     LS        length of SHORT
C     MXSWP     maximum possible swaps
C     NO        number of STR1 characters found in STR2
C     ORDER     index in shorter string of a character in longer
C     SHORT     the shorter of the input strings
C     STR1      first input string
C     STR2      second input string
C     SWAPS     swaps made
C     TEMP      temporary for sorting
C
C     formal parameters
      REAL*8 STRSIM
      CHARACTER*1 STR1(L1),STR2(L2)
C
C     local variables
      PARAMETER(LMAX=24)
      CHARACTER*1 LONG(LMAX),SHORT(LMAX)
      INTEGER*4 ORDER(LMAX),TEMP,SWAPS
C
C ------------------------------------------------------------------
C
C     sanity-check the string lengths
      IF(L1.LE.0 .OR. L1.GT.LMAX .OR. L2.LE.0 .OR. L2.GT.LMAX) THEN
         STRSIM=-1.D0
         RETURN
      ENDIF
C
C     copy the longer string to LONG and the shorter to SHORT
      IF(L1.GE.L2) THEN
         DO 1 K=1,L1
              LONG(K)=STR1(K)
              IF(K.LE.L2) SHORT(K)=STR2(K)
    1    CONTINUE
         LS=L2
         LL=L1
      ELSE
         DO 2 K=1,L2
              LONG(K)=STR2(K)
              IF(K.LE.L1) SHORT(K)=STR1(K)
    2    CONTINUE
         LS=L1
         LL=L2
      ENDIF
C
C     single characters can just be compared
      IF(LL.EQ.1) THEN
         IF(STR1(1).EQ.STR2(1)) THEN
            STRSIM=1.D0
         ELSE
            STRSIM=0.D0
         ENDIF
         RETURN
      ENDIF
C
C     an order of zero means no match was found anywhere
      DO 3 K=1,LL
           ORDER(K)=0
    3 CONTINUE
      NO=0
C
C     for each character in the shorter string
      DO 4 KS=1,LS
C          search for it in the longer string
           DO 5 KL=1,LL
                IF(SHORT(KS).EQ.LONG(KL) .AND.
     ;             ORDER(KL).EQ.0) THEN
C                  record index in short string of match in long
                   ORDER(KL)=KS
                   NO=NO+1
                   GO TO 4
                ENDIF
    5      CONTINUE
    4 CONTINUE
C
C     sort the indices in ascending order
      SWAPS=0
      MXSWP=0
      DO 6 K=1,LL-1
           DO 7 KK=K+1,LL
                MXSWP=MXSWP+1
                IF(ORDER(K).EQ.0 .OR. ORDER(KK).EQ.0) THEN
                   SWAPS=SWAPS+1
                   GO TO 7
                ENDIF
                IF(ORDER(K).GT.ORDER(KK)) THEN
                   TEMP=ORDER(K)
                   ORDER(K)=ORDER(KK)
                   ORDER(KK)=TEMP
                   SWAPS=SWAPS+1
                   GO TO 7
                ENDIF
    7      CONTINUE
    6 CONTINUE
C
C     compute the similarity score
      STRSIM=1.D0-DFLOAT(SWAPS)/DFLOAT(MXSWP+NO)
      RETURN
      END
