C
Code by Michael Kupferschmid after Mark Loose
C
      FUNCTION STRUVE(X,NU, RC)
C     This routine returns the modified Struve function of order NU
C     at the real argument X, as computed from Formula 12.2.1 in
C     "Handbook of Mathematical Functions" by Abramowitz and Stegun.
C
C     variable  meaning
C     --------  -------
C     DABS      Fortran function returns |REAL*8|
C     DBLE      Fortran function returns REAL*8 for REAL*4
C     DGAMMA    Fortran function returns gamma function of a REAL*8
C     G1        gamma(Y1)
C     G2        gamma(Y2)
C     K         index on the terms in the sum
C     KMAX      limit on K
C     NU        integer order of the function
C     OLD       value of SUM at previous iteration
C     RC        return code; 0 => all went well
C     SUM       sum in the formula
C     TOL       convergence tolerance
C     X         real argument of the function
C     Y1        gamma function argument for G1
C     Y2        gamma function argument for G2
C
C     formal parameters
      REAL*8 STRUVE,X
      INTEGER*4 RC
C
C     set limits on the infinite sum
      INTEGER*4 KMAX/20/
      REAL*8 TOL/1.D-12/
C
C     other local variables
      REAL*8 G1,G2,Y1,Y2,SUM,OLD
C
C ------------------------------------------------------------------
C
C     initialize
      RC=1
      SUM=0.D0
C
C     approximate the infinite sum
      K=-1
    1 K=K+1
      IF(K.GT.KMAX) RETURN
         Y1=1.5D0+DBLE(K)
         G1=DGAMMA(Y1)
         Y2=Y1+DBLE(NU)
         G2=DGAMMA(Y2)
         OLD=SUM
         SUM=SUM+(0.5D0*X)**(2*K)/(G1*G2)
      IF(DABS(SUM-OLD).GT.TOL) GO TO 1
C
C     return the function value
      STRUVE=SUM*(0.5D0*X)**(NU+1)
      RC=0
      RETURN
      END
