C
Code by Michael Kupferschmid
C     
      FUNCTION STRWID(STRING,LS,PT)
C     This routine returns the REAL*4 width in points of STRING,
C     assuming it will be set in Computer Modern at PT points.
C
C     variable  meaning
C     --------  -------
C     DIGIT     1234567890
C     FLOAT     Fortran function returns REAL*4 for INTEGER*4
C     I         index on the characters in a list
C     K         index on the characters in STRING
C     LOWR      abcdefghijklmnopqrstuvwxyz
C     LS        number of characters in STRING
C     PT        point size of type
C     PUNC       .,;:!?()[]-+*/="@#$%^&_{}|\'
C     STRING    the string whose width is to be estimated
C     UPER      ABCDEFGHIJKLMNOPQRSTUVWXYZ
C     WDIGIT    width of a digit (they are all the same width)
C     WLOWR     widths of lower-case alphabetic characters
C     WPUNC     widths of punctuation characters
C     WTOT      total width of STRING in points
C     WUPER     widths of upper-case alphabetic characters
C
C     formal parameters
      CHARACTER*1 STRING(LS)
      REAL*4 PT
C
C     characters and their widths in [points per 50 characters]
      CHARACTER*1 LOWR(26)/'a','b','c','d','e','f','g','h','i','j',
     ;                     'k','l','m','n','o','p','q','r','s','t',
     ;                     'u','v','w','x','y','z'/
      INTEGER*4 WLOWR(26)/293,326,262,326,262,172,293,326,162,178,
     ;                    308,162,490,326,308,326,308,228,231,228,
     ;                    326,308,423,308,308,262/
      CHARACTER*1 UPER(26)/'A','B','C','D','E','F','G','H','I','J',
     ;                     'K','L','M','N','O','P','Q','R','S','T',
     ;                     'U','V','W','X','Y','Z'/
      INTEGER*4 WUPER(26)/440,415,424,448,399,383,459,440,227,301,
     ;                    455,367,538,440,455,399,455,432,326,424,
     ;                    440,440,603,440,440,358/
C
C     guessing about #$%^&_{}|\' as these are harder to measure
      CHARACTER*1 PUNC(30)/' ','.',',',';',':','!','?','(',')','[',
     ;                     ']','-','+','*','/','=','"','@',
     ;                     '#','$','%','^','&','_','{','}','|','\',
     ;                     "'",'`'/
      INTEGER*4 WPUNC(30)/196,162,162,162,162,162,277,228,228,162,
     ;                    162,194,456,292,293,456,292,456,
     ;                    456,456,277,292,277,456,162,162,162,293,
     ;                    146,146/
C
C     digits and their widths [points per 50 characters]
      CHARACTER*1 DIGIT(10)/'1','2','3','4','5','6','7','8','9','0'/
      INTEGER*4 WDIGIT/293/
C
C     other local variable
      INTEGER*4 WTOT
C
C ------------------------------------------------------------------
C
      WTOT=0
      DO 1 K=1,LS
           DO 2 I=1,26
                IF(STRING(K).EQ.LOWR(I)) THEN
                   WTOT=WTOT+WLOWR(I)
                   GO TO 1
                ENDIF
    2      CONTINUE
           DO 3 I=1,26
                IF(STRING(K).EQ.UPER(I)) THEN
                   WTOT=WTOT+WUPER(I)
                   GO TO 1
                ENDIF
    3      CONTINUE
           DO 4 I=1,30
                IF(STRING(K).EQ.PUNC(I)) THEN
                   WTOT=WTOT+WPUNC(I)
                   GO TO 1
                ENDIF
    4      CONTINUE
           DO 5 I=1,10
                IF(STRING(K).EQ.DIGIT(I)) THEN
                   WTOT=WTOT+WDIGIT
                   GO TO 1
                ENDIF
    5      CONTINUE
C
C          report that the character was not found
           STRWID=-FLOAT(K)
           RETURN
    1 CONTINUE
      STRWID=(PT/12.0)*(FLOAT(WTOT)/50.0)
      RETURN
      END
