C
Code by Michael Kupferschmid 
C
      SUBROUTINE TBLBIN(DATA,LDATA,NPTS,X, LAST )
C     This routine finds the row in DATA to use in computing
C     the value at X of the function defined by the table.
C
C     variable  meaning
C     --------  -------
C     DATA      table defining the function
C     K         counter on bisection iterations
C     LAST      bin used on previous call; bin used this time 
C     LDATA     leading dimension of DATA
C     LEFT      left bracket on the bin containing X
C     MAX0      Fortran function for larger of INTEGER*4s
C     MIN0      Fortran function for smaller of INTEGER*4s
C     NPTS      the DATA table contains NPTS points, NPTS-1 bins
C     RIGHT     right bracket on the bin containing X
C     X         value of the function that the bin should contain 
C
      REAL*8 DATA(LDATA,*),X
      INTEGER*4 RIGHT
C
C ------------------------------------------------------------------
C
C     make sure LAST has an interior value
      LAST=MAX0(1,MIN0(NPTS-1,LAST))
C
C     see if X is in the bin LAST referenced for this function
      IF(X.LT.DATA(LAST,  1)) GO TO 1
      IF(X.GT.DATA(LAST+1,1)) GO TO 2
C     yes; return the value unchanged 
      RETURN 
C
C     the bin we want is to the left
    1 RIGHT=LAST
      LEFT=1
      GO TO 3 
C
C     the bin we want is to the right
    2 LEFT=LAST+1
      RIGHT=NPTS-1
C
C     check for endpoints
    3 IF(X.GT.DATA(1,   1)) GO TO 4
         LAST=0
         RETURN
    4 IF(X.LT.DATA(NPTS,1)) GO TO 5
         LAST=NPTS
         RETURN
C
C     bisect to find the correct bin
    5 DO 6 K=1,NPTS
           LAST=(LEFT+RIGHT)/2
           IF(X.GE.DATA(LAST,  1)) GO TO 7
              RIGHT=LAST
              GO TO 6
    7      IF(X.LE.DATA(LAST+1,1)) RETURN 
              LEFT=LAST+1
    6 CONTINUE
C
C     convergence should always be achieved in NPTS iterations
      WRITE(0,991)
  991 FORMAT('TBLBIN: bisection failed to converge')
      STOP
      END
